#-*- coding: utf-8 -*-

import sys
from optparse import OptionParser

from django.conf import settings, global_settings

if not settings.configured:
    settings.configure(
        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
            }
        },
        INSTALLED_APPS=[
            "tests",
        ],
        TEMPLATE_CONTEXT_PROCESSORS=global_settings.TEMPLATE_CONTEXT_PROCESSORS +
            ("django.core.context_processors.request",),
        ROOT_URLCONF="tests.urls",
        DEBUG=False,
    )

from django.test.simple import DjangoTestSuiteRunner


def run_tests(*test_args, **kwargs):
    if not test_args:
        test_args = ["tests"]

    test_runner = DjangoTestSuiteRunner(
        verbosity=kwargs.get("verbosity", 1),
        interactive=kwargs.get("interactive", False),
        failfast=kwargs.get("failfast")
    )
    failures = test_runner.run_tests(test_args)
    sys.exit(failures)


if __name__ == "__main__":
    parser = OptionParser()
    parser.add_option("--failfast", action="store_true", default=False, dest="failfast")
    parser.add_option("--verbosity", action="store", default=1, type=int, dest="verbosity")
    (options, args) = parser.parse_args()
    run_tests(failfast=options.failfast, verbosity=options.verbosity, *args)



#-*- coding: utf-8 -*-

import datetime
import json

from django.test import TestCase
from django.core.urlresolvers import reverse

from django.utils import timezone

from models import Article
from infinite_scroll_pagination.paginator import SeekPaginator


class PaginatorTest(TestCase):

    def setUp(self):
        date = timezone.now()

        for i in range(25):
            seconds = datetime.timedelta(seconds=i)
            Article.objects.create(title="%s" % i, date=date, date_unique=date + seconds)

        self.paginator = SeekPaginator(Article.objects.all(), per_page=10, lookup_field="date_unique")

    def test_prepare_order(self):
        self.assertListEqual(self.paginator.prepare_order(), ["-date_unique", "-pk"])
        self.paginator.lookup_field = "pk"
        self.assertListEqual(self.paginator.prepare_order(), ["-pk", ])
        self.paginator.lookup_field = "id"
        self.assertListEqual(self.paginator.prepare_order(), ["-id", ])

    def test_prepare_lookup(self):
        lookup_f, lookup_e = self.paginator.prepare_lookup(value=1, pk=2)
        self.assertDictEqual(lookup_f, {"date_unique__lte": 1, })
        self.assertDictEqual(lookup_e, {"date_unique": 1, "pk__gte": 2})

        self.paginator.lookup_field = "pk"
        lookup_f, lookup_e = self.paginator.prepare_lookup(value=2, pk=2)
        self.assertDictEqual(lookup_f, {"pk__lt": 2, })
        self.assertIsNone(lookup_e)

        self.paginator.lookup_field = "id"
        lookup_f, lookup_e = self.paginator.prepare_lookup(value=2, pk=2)
        self.assertDictEqual(lookup_f, {"id__lt": 2, })
        self.assertIsNone(lookup_e)

    def test_paginator(self):
        articles = Article.objects.all().order_by("-date_unique")

        page_1 = self.paginator.page()
        self.assertListEqual(list(page_1), list(articles[:10]))

        page_2 = self.paginator.page(value=page_1[-1].date_unique, pk=page_1[-1].pk)
        self.assertListEqual(list(page_2), list(articles[10:20]))

        page_3 = self.paginator.page(value=page_2[-1].date_unique, pk=page_2[-1].pk)
        self.assertListEqual(list(page_3), list(articles[20:]))

    def test_first_page(self):
        page = self.paginator.page()
        self.assertTrue(page.has_next())

    def test_last_page(self):
        articles = Article.objects.all().order_by("-date_unique")
        next_to_last = list(articles)[-2]

        page = self.paginator.page(value=next_to_last.date_unique, pk=next_to_last.pk)
        self.assertFalse(page.has_next())

    def test_lookup_not_unique(self):
        self.paginator.lookup_field = "date"
        articles = Article.objects.all().order_by("-date", "-pk")

        page_1 = self.paginator.page()
        self.assertListEqual(list(page_1), list(articles[:10]))

        page_2 = self.paginator.page(value=page_1[-1].date, pk=page_1[-1].pk)
        self.assertListEqual(list(page_2), list(articles[10:20]))

    def test_lookup_pk(self):
        self.paginator.lookup_field = "pk"
        articles = Article.objects.all().order_by("-pk")

        page_1 = self.paginator.page()
        self.assertListEqual(list(page_1), list(articles[:10]))

        page_2 = self.paginator.page(value=page_1[-1].pk, pk=page_1[-1].pk)
        self.assertListEqual(list(page_2), list(articles[10:20]))

    def test_lookup_id(self):
        self.paginator.lookup_field = "id"
        articles = Article.objects.all().order_by("-id")

        page_1 = self.paginator.page()
        self.assertListEqual(list(page_1), list(articles[:10]))

        page_2 = self.paginator.page(value=page_1[-1].id, pk=page_1[-1].id)
        self.assertListEqual(list(page_2), list(articles[10:20]))

    def test_reverse_date_for_pk(self):
        """
        When the date increment does not match the pk increment,
        we should still get the right results.
        """
        Article.objects.all().delete()

        # asc order date and desc order pk
        date = timezone.now()
        dates = reversed([date + datetime.timedelta(seconds=seconds)
                          for seconds in range(25)])

        for i, d in enumerate(dates):
            Article.objects.create(title="%s" % i, date=date, date_unique=d)

        articles = Article.objects.all().order_by("-date_unique")

        page_1 = self.paginator.page()
        self.assertListEqual(list(page_1), list(articles[:10]))

        page_2 = self.paginator.page(value=page_1[-1].date_unique, pk=page_1[-1].pk)
        self.assertListEqual(list(page_2), list(articles[10:20]))

        page_3 = self.paginator.page(value=page_2[-1].date_unique, pk=page_2[-1].pk)
        self.assertListEqual(list(page_3), list(articles[20:]))


class PageTest(TestCase):

    def setUp(self):
        date = timezone.now()

        for i in range(25):
            seconds = datetime.timedelta(seconds=i)
            Article.objects.create(title="%s" % i, date=date, date_unique=date + seconds)

        self.paginator = SeekPaginator(Article.objects.all(), per_page=10, lookup_field="date_unique")

    def test_unimplemented(self):
        page = self.paginator.page()
        self.assertRaises(NotImplementedError, page.has_previous)
        self.assertRaises(NotImplementedError, page.next_page_number)
        self.assertRaises(NotImplementedError, page.previous_page_number)
        self.assertRaises(NotImplementedError, page.start_index)
        self.assertRaises(NotImplementedError, page.end_index)

    def test_objects_left(self):
        articles = Article.objects.all().order_by("-date_unique")
        page = self.paginator.page()
        self.assertEqual(page.objects_left, len(articles[self.paginator.per_page:]))

        # last page
        art = list(articles)[-self.paginator.per_page]
        page_last = self.paginator.page(value=art.date_unique, pk=art.pk)
        self.assertEqual(page_last.objects_left, 0)

    def test_pages_left(self):
        page = self.paginator.page()
        self.assertEqual(page.pages_left, 2)

    def test_next_page_pk(self):
        page = self.paginator.page()
        self.assertEqual(page.next_page_pk(), page[-1].pk)


class PaginatorViewTest(TestCase):

    def setUp(self):
        date = timezone.now()

        for i in range(25):
            seconds = datetime.timedelta(seconds=i)
            Article.objects.create(title="%s" % i, date=date, date_unique=date + seconds)

    def test_first_page(self):
        response = self.client.get(reverse('pagination-ajax'),
                                   HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        res = json.loads(response.content)
        articles = Article.objects.all().order_by("-date", "-pk")
        self.assertEqual(res['articles'], [{u'title': a.title, } for a in articles[:20]])

    def test_page(self):
        articles = Article.objects.all().order_by("-date", "-pk")
        art = articles[20]

        response = self.client.get(reverse('pagination-ajax', kwargs={'pk': str(art.pk), }),
                                   HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        res = json.loads(response.content)
        self.assertEqual(res['articles'], [{u'title': a.title, } for a in articles[21:40]])


#-*- coding: utf-8 -*-

from math import ceil

from django.core.paginator import EmptyPage, Page


__all__ = ["SeekPaginator", "SeekPage", "EmptyPage"]


class SeekPaginator(object):

    def __init__(self, query_set, per_page, lookup_field):
        self.query_set = query_set
        self.per_page = per_page
        self.lookup_field = lookup_field

    def prepare_order(self):
        lookup_field_desc = "-%s" % self.lookup_field

        if self.lookup_field not in ("pk", "id"):
            return [lookup_field_desc, "-pk"]
        else:
            return [lookup_field_desc, ]

    def prepare_lookup(self, value, pk):
        if self.lookup_field not in ("pk", "id"):
            lookup = "%s__lte" % self.lookup_field
            lookup_filter = {lookup: value, }
            lookup_exclude = {self.lookup_field: value, "pk__gte": pk, }
            return lookup_filter, lookup_exclude
        else:
            lookup = "%s__lt" % self.lookup_field
            lookup_filter = {lookup: value, }
            lookup_exclude = None
            return lookup_filter, lookup_exclude

    def page(self, value=None, pk=None):
        if (value is None and pk is not None) or (value is not None and pk is None):
            raise ValueError("Both 'value' and 'pk' arguments must be provided")

        query_set = self.query_set

        if value is not None and pk is not None:
            lookup_filter, lookup_exclude = self.prepare_lookup(value, pk)
            query_set = query_set.filter(**lookup_filter)

            if lookup_exclude:
                query_set = query_set.exclude(**lookup_exclude)

        order = self.prepare_order()
        query_set = query_set.order_by(*order)[:self.per_page + 1]

        object_list = list(query_set)
        has_next = len(object_list) > self.per_page
        object_list = object_list[:self.per_page]

        if not object_list and value:
            raise EmptyPage("That page contains no results")

        return SeekPage(object_list=object_list, number=value, paginator=self, has_next=has_next)


class SeekPage(Page):

    def __init__(self, object_list, number, paginator, has_next):
        super(SeekPage, self).__init__(object_list, number, paginator)
        self._has_next = has_next
        self._objects_left = None
        self._pages_left = None

    def __repr__(self):
        return '<Page value %s>' % self.number or ""

    def has_next(self):
        return self._has_next

    def has_previous(self):
        raise NotImplementedError

    def has_other_pages(self):
        return self.has_next()

    def next_page_number(self):
        raise NotImplementedError

    def previous_page_number(self):
        raise NotImplementedError

    def start_index(self):
        raise NotImplementedError

    def end_index(self):
        raise NotImplementedError

    @property
    def objects_left(self):
        """
        Returns the total number of *objects* left
        """
        if not self.has_next():
            return 0

        if self._objects_left is None:
            last = self.object_list[-1]
            value = getattr(last, self.paginator.lookup_field)
            lookup_filter, lookup_exclude = self.paginator.prepare_lookup(value, last.pk)
            query_set = self.paginator.query_set.filter(**lookup_filter)

            if lookup_exclude:
                query_set = query_set.exclude(**lookup_exclude)

            order = self.paginator.prepare_order()
            self._objects_left = query_set.order_by(*order).count()

        return self._objects_left

    @property
    def pages_left(self):
        """
        Returns the total number of *pages* left
        """
        if not self.objects_left:
            return 0

        if self._pages_left is None:
            self._pages_left = int(ceil(self.objects_left / float(self.paginator.per_page)))

        return self._pages_left

    def next_page_pk(self):
        return self.object_list[-1].pk


#-*- coding: utf-8 -*-

from django.conf.urls import patterns, url

urlpatterns = patterns("tests.views",
    url(r'^page/$', 'pagination_ajax', name='pagination-ajax'),
    url(r'^page/(?P<pk>\d+)/$', 'pagination_ajax', name='pagination-ajax'),
)


#-*- coding: utf-8 -*-

import json

from django.http import Http404, HttpResponse
from django.shortcuts import get_object_or_404

from infinite_scroll_pagination.paginator import SeekPaginator, EmptyPage

from models import Article


def pagination_ajax(request, pk=None):
    if not request.is_ajax():
        return Http404()

    if pk is not None:
        date = get_object_or_404(Article, pk=pk).date
    else:
        date = None

    articles = Article.objects.all()
    paginator = SeekPaginator(articles, per_page=20, lookup_field="date")

    try:
        page = paginator.page(value=date, pk=pk)
    except EmptyPage:
        return Http404()

    articles_list = [{"title": a.title, } for a in page]
    data = {'articles': articles_list,
            'has_next': page.has_next(),
            'pk': page[-1].pk}

    return HttpResponse(json.dumps(data), content_type="application/json")


#-*- coding: utf-8 -*-

from django.db import models


class Article(models.Model):

    title = models.CharField(max_length=75)
    date = models.DateTimeField()
    date_unique = models.DateTimeField(unique=True)

    def __unicode__(self):
        return self.title